# GUI

from inc_noesis import *

import noesis
import rapi
import os
import hashlib
import json
import sys

from ctypes import *
import noewin

import nrfile
import nrtools
import nrdump
import nrnoeimp

import ctypes


def writeLog(s):
    print(s)

def makeWPARAM(par0, par1):
    return (((par0 & 0xFFFF) << 16) | (par1 & 0xFFFF))

def makeLPARAM(par0, par1):
    return (((par0 & 0xFFFF) << 16) | (par1 & 0xFFFF))

def selChangCombo(comboBox):
     ctrlId = comboBox.controlId
     return noewin.user32.PostMessageW(comboBox.noeParentWnd.hWnd, noewin.WM_COMMAND, makeWPARAM(int(noewin.CBN_SELCHANGE), ctrlId), comboBox.hWnd)


ripDirSet = False
def setRipperDirectory():
    global ripDirSet
    if not ripDirSet:
        ripDirSet = True
        nrnoeimp.setRipperDir()


class ControlGroupBase:
    def __init__(self, noeWnd, xpos, ypos):
        self.noeWnd = noeWnd
        self.xpos = xpos
        self.ypos = ypos
        self.childs = []

    def regCtrl(self, child):
        self.childs.append(child)

    def destroy(self):
        for child in self.childs:
            if isinstance(child, int):
                # Destroy single Win32-control
                ctrl = self.noeWnd.getControlByIndex(child)
                if ctrl:
                    noewin.user32.DestroyWindow(ctrl.hWnd)
                else:
                    writeLog("Control with idx={:d} not found".format(child))
            elif isinstance(child, ControlGroupBase):
                # Destroy child groups
                child.destroy()
            elif isinstance(child, noewin.NoeUserControlBase):
                # Destroy single Win32-control
                ctrl = self.noeWnd.getControlById(child.controlId)
                if ctrl:
                    noewin.user32.DestroyWindow(ctrl.hWnd)
                else:
                    writeLog("Control with controlId={:d} not found".format(controlId))

#####################################################################################################

UDS_WRAP          = 0x0001
UDS_SETBUDDYINT   = 0x0002
UDS_ALIGNRIGHT    = 0x0004
UDS_ALIGNLEFT     = 0x0008
UDS_AUTOBUDDY     = 0x0010
UDS_ARROWKEYS     = 0x0020
UDS_HORZ          = 0x0040
UDS_NOTHOUSANDS   = 0x0080
UDS_HOTTRACK      = 0x0100

WM_USER         = 0x400
UDM_SETRANGE    = (WM_USER+101)
UDM_GETRANGE    = (WM_USER+102)
UDM_SETPOS      = (WM_USER+103)
UDM_GETPOS      = (WM_USER+104)
UDM_SETBUDDY    = (WM_USER+105)
UDM_GETBUDDY    = (WM_USER+106)
UDM_SETACCEL    = (WM_USER+107)
UDM_GETACCEL    = (WM_USER+108)
UDM_SETBASE     = (WM_USER+109)
UDM_GETBASE     = (WM_USER+110)
UDM_SETRANGE32  = (WM_USER+111)
UDM_GETRANGE32  = (WM_USER+112)
UDM_SETPOS32    = (WM_USER+113)
UDM_GETPOS32    = (WM_USER+114)

EM_GETSEL               = 0x00B0
EM_SETSEL               = 0x00B1
EM_GETRECT              = 0x00B2
EM_SETRECT              = 0x00B3
EM_SETRECTNP            = 0x00B4
EM_SCROLL               = 0x00B5
EM_LINESCROLL           = 0x00B6
EM_SCROLLCARET          = 0x00B7
EM_GETMODIFY            = 0x00B8
EM_SETMODIFY            = 0x00B9
EM_GETLINECOUNT         = 0x00BA
EM_LINEINDEX            = 0x00BB
EM_SETHANDLE            = 0x00BC
EM_GETHANDLE            = 0x00BD
EM_GETTHUMB             = 0x00BE
EM_LINELENGTH           = 0x00C1
EM_REPLACESEL           = 0x00C2
EM_GETLINE              = 0x00C4
EM_LIMITTEXT            = 0x00C5
EM_CANUNDO              = 0x00C6
EM_UNDO                 = 0x00C7
EM_FMTLINES             = 0x00C8
EM_LINEFROMCHAR         = 0x00C9
EM_SETTABSTOPS          = 0x00CB
EM_SETPASSWORDCHAR      = 0x00CC
EM_EMPTYUNDOBUFFER      = 0x00CD
EM_GETFIRSTVISIBLELINE  = 0x00CE
EM_SETREADONLY          = 0x00CF
EM_SETWORDBREAKPROC     = 0x00D0
EM_GETWORDBREAKPROC     = 0x00D1
EM_GETPASSWORDCHAR      = 0x00D2


def defaultCommandMethod(noeWnd, controlId, wParam, lParam):
    return False


class SpinBoxBuddyCtrl(noewin.NoeUserControlBase):
    def __init__(self, noeParentWnd, name, x, y, width, height, commandMethod):
        super().__init__(noeParentWnd, noeParentWnd.currentControlId, x, y, width, height, commandMethod)
        self.exStyle= noewin.WS_EX_LEFT | noewin.WS_EX_CLIENTEDGE | noewin.WS_EX_CONTEXTHELP
        self.name   = name
        self.style  = noewin.WS_CHILDWINDOW | noewin.WS_VISIBLE | noewin.WS_BORDER | noewin.ES_NUMBER | noewin.ES_LEFT
        self.hWnd   = noewin.user32.CreateWindowExW(self.exStyle, "Edit", self.name, self.style, x, y, width, height, noeParentWnd.hWnd, self.controlId, noeParentWnd.hInst, 0)

    def getText(self):
        l = noewin.user32.GetWindowTextLengthW(self.hWnd) + 1
        textBuffer = ctypes.create_unicode_buffer(l)
        noewin.user32.GetWindowTextW(self.hWnd, textBuffer, l)
        return textBuffer.value
    def setText(self, text):
        noewin.user32.SetWindowTextW(self.hWnd, text)

class SpinBoxCtrl(noewin.NoeUserControlBase):
    def __init__(self, noeParentWnd, name, x, y, width, height, minVal, maxVal, horiz, defVal, commandMethod):
        super().__init__(noeParentWnd, noeParentWnd.currentControlId, x, y, width, height, commandMethod)
        self.name = name
        self.exStyle = noewin.WS_EX_LEFT | noewin.WS_EX_LTRREADING
        self.style = noewin.WS_CHILDWINDOW | noewin.WS_VISIBLE | UDS_AUTOBUDDY | UDS_SETBUDDYINT | UDS_ALIGNRIGHT | UDS_ARROWKEYS | UDS_HOTTRACK
        if horiz:
            self.style = self.style | UDS_HORZ

        self.hWnd = noewin.user32.CreateWindowExW(self.exStyle, "msctls_updown32", self.name, self.style, x, y, width, height, noeParentWnd.hWnd, self.controlId, noeParentWnd.hInst, 0)
        noewin.user32.SendMessageW(self.hWnd, UDM_SETRANGE, 0, makeLPARAM(minVal, maxVal))
        noewin.user32.SendMessageW(self.hWnd, UDM_SETPOS32, 0, makeLPARAM(0, defVal))

# SpinBox control (Label+Edit+UpDown)
class SpinBoxGroup(ControlGroupBase):
    def __init__(self, noeWnd, name, xpos, ypos, width, height, minVal, maxVal, horiz, defVal, commandMethod, offs1):
        super().__init__(noeWnd, xpos, ypos)

        offs = 60
        if offs1:
            offs = offs1

        # Label
        self.labelIdx = noeWnd.createStatic(name, xpos + 0, ypos + 5, offs, 20)
        self.regCtrl(self.labelIdx)

        # Edit
        self.edit   = SpinBoxBuddyCtrl(noeWnd, str(defVal), self.xpos + offs, self.ypos + 0, 70, 26, commandMethod)
        noeWnd.addControl(self.edit)
        self.regCtrl(self.edit)

        # Up-Down
        self.spin   = SpinBoxCtrl(noeWnd, str(defVal), self.xpos + offs , self.ypos + 0, 50, 26, minVal, maxVal, horiz, defVal, commandMethod)
        spinIdx = noeWnd.addControl(self.spin)
        self.regCtrl(self.spin)
    def getText(self):
        return self.edit.getText()
    def setText(self, text):
        return self.edit.setText(text)

def createSpinBoxGroup(noeWnd, label, x, y, width, height, minVal, maxVal, horiz, defVal, commandMethod = defaultCommandMethod, offs = None):
    if noeWnd.hWnd:
        spinBoxGrp = SpinBoxGroup(noeWnd, label, x, y, width, height, minVal, maxVal, horiz, defVal, commandMethod, offs)
        return spinBoxGrp
    return None

##########################################################

class EditLineCtrl(noewin.NoeUserControlBase):
    def __init__(self, noeParentWnd, name, x, y, width, height, commandMethod = defaultCommandMethod):
        super().__init__(noeParentWnd, noeParentWnd.currentControlId, x, y, width, height, commandMethod)
        self.exStyle= noewin.WS_EX_LEFT | noewin.WS_EX_CLIENTEDGE | noewin.WS_EX_CONTEXTHELP 
        self.name   = name
        self.style  = noewin.WS_CHILDWINDOW | noewin.WS_VISIBLE | noewin.WS_BORDER | noewin.ES_AUTOHSCROLL
        self.hWnd   = noewin.user32.CreateWindowExW(self.exStyle, "Edit", self.name, self.style, x, y, width, height, noeParentWnd.hWnd, self.controlId, noeParentWnd.hInst, 0)
        self.setText(str(name))
    def getText(self):
        l = noewin.user32.GetWindowTextLengthW(self.hWnd) + 1
        textBuffer = ctypes.create_unicode_buffer(l)
        noewin.user32.GetWindowTextW(self.hWnd, textBuffer, l)
        return textBuffer.value
    def setText(self, text):
        noewin.user32.SetWindowTextW(self.hWnd, text)
##########################################################


# LabledEdit control (Static+Edit)
class LabledEditGroup(ControlGroupBase):
    def __init__(self, noeWnd, label, defVal, xpos, ypos, width, height, offs1 = None):
        super().__init__(noeWnd, xpos, ypos)

        offs = 60
        if offs1:
            offs = offs1

        # Label
        self.labelIdx = noeWnd.createStatic(label, xpos + 0, ypos + 5, offs, 20)
        self.regCtrl(self.labelIdx)
        # Edit
        #self.editIdx = noeWnd.createEditBox(xpos + 60, ypos, width, height, str(defVal), None, False)
        self.edit = EditLineCtrl(noeWnd, defVal, xpos + offs, ypos, width, height)
        noeWnd.addControl(self.edit)
        self.regCtrl(self.edit)
    def getText(self):
        return self.edit.getText()

#####################################################################################################

# Get import parametes and reload model/scene

def isAdvancedViewApiPresent():
    if noesis.getAPIVersion() >= 82:   # Noesis version >= 4.461
        return True

    return False


def reloadWithView(fileName):
    #writeLog("View load")
    if noesis.isPreviewModuleRAPIValid():
        noesis.setPreviewModuleRAPI()
        #you could also use rapi.getOpenPreviewFile() here to determine whether to preserve the view across loads based on the currently open filename
        viewData = rapi.getInternalViewData()
    else:
        viewData = None
    noesis.openFile(fileName)
    if viewData:
        noesis.setPreviewModuleRAPI() #the preview module was invalidated when we loaded a new scene, so set it again
        rapi.setInternalViewData(viewData)


def reload3dModel(noeWnd):
    options = nrnoeimp.getImportOptions()
    retreiveOptionsFromGui(noeWnd, options)
    fileName = options.get('fileName', None)
    if not fileName:
        return

    if not isAdvancedViewApiPresent():
        noesis.openFile(fileName)
    else:
        reloadWithView(fileName)


def updateButtonMethod(noeWnd, controlId, wParam, lParam):
    reload3dModel(noeWnd)
    return True


class UpControlGroup(ControlGroupBase):
    def __init__(self, noeWnd, xpos, ypos):
        super().__init__(noeWnd, xpos, ypos)
        self.updateButtonIdx   = noeWnd.createButton("Update", self.xpos + 0, self.ypos + 0, 96, 32, updateButtonMethod, True)
        self.autoReloadFlagIdx = noeWnd.createCheckBox("Auto Reload on FOV change", self.xpos + 120, self.ypos + 0, 216, 32)
        self.regCtrl(self.updateButtonIdx)
        self.regCtrl(self.autoReloadFlagIdx)


class UpControlGroupLocal(ControlGroupBase):
    def __init__(self, noeWnd, xpos, ypos):
        super().__init__(noeWnd, xpos, ypos)
        self.updateButtonIdx   = noeWnd.createButton("Update", self.xpos + 0, self.ypos + 0, 96, 32, updateButtonMethod, True)
        self.regCtrl(self.updateButtonIdx)

def fovSpinBoxMethodCallback(noeWnd, controlId, wParam, lParam):
    notificationId = (wParam >> 16)
    #writeLog("{:d}".format(notificationId))

    if hasattr(noeWnd, 'currentGeometryParamsDlg'):
        if isinstance(noeWnd.currentGeometryParamsDlg, GeometryParamsManualGroup):
            if noeWnd.currentGeometryParamsDlg.spinBoxGrp.edit.controlId == controlId:
                options = nrnoeimp.getImportOptions()
                retreiveOptionsFromGui(noeWnd, options)
                if options['autoReload']:
                    reload3dModel(noeWnd)

class GeometryParamsManualGroup(ControlGroupBase):
    def __init__(self, noeWnd, xpos, ypos):
        super().__init__(noeWnd, xpos, ypos)
        self.spinBoxGrp = createSpinBoxGroup(noeWnd, "FOV_Y:", xpos + 0, ypos + 0, 70, 32, 0, 180, False, 45, fovSpinBoxMethodCallback)
        self.regCtrl(self.spinBoxGrp)

        self.widthEdit = LabledEditGroup(noeWnd, "Width", "1024", xpos + 0, ypos + 32, 100, 28)
        self.regCtrl(self.widthEdit)

        self.heightEdit = LabledEditGroup(noeWnd, "Height", "768", xpos + 0, ypos + 64, 100, 28)
        self.regCtrl(self.heightEdit)

        self.nearEdit = LabledEditGroup(noeWnd, "Near", "0.01", xpos + 0, ypos + 100, 100, 28)
        self.regCtrl(self.nearEdit)

        self.farEdit = LabledEditGroup(noeWnd, "Far", "1000", xpos + 0, ypos + 132, 100, 28)
        self.regCtrl(self.farEdit)

    def calcProjMat(self):
        fov      = float(self.spinBoxGrp.getText())
        width    = float(self.widthEdit.getText())
        height   = float(self.heightEdit.getText())
        nearDist = float(self.nearEdit.getText())
        farDist  = float(self.farEdit.getText())
        return nrtools.perspMatFovLH(fov, width, height, nearDist, farDist)

class GeometryParamsMatrixGroup(ControlGroupBase):
    def __init__(self, noeWnd, xpos, ypos):
        super().__init__(noeWnd, xpos, ypos)
        self.edit = LabledEditGroup(noeWnd, "Matrix:", "[[1, 0, 0, 0], [0, 1, 0, 0], [0, 0, 1, 0],[0, 0, 0, 1]]", xpos + 0, ypos + 0, 600, 28)
        self.regCtrl(self.edit)
        self.checkboxIdx = noeWnd.createCheckBox("Transpose", self.xpos + 0, self.ypos + 32, 300, 32)
        self.regCtrl(self.checkboxIdx)
    def getMatrixStr(self):
        return self.edit.getText()
    def isTransposeChecked(self):
        return self.noeWnd.getControlByIndex(self.checkboxIdx).isChecked()


def geometryParametersComboMethod(noeWnd, controlId, wParam, lParam):
    notificationId = (wParam >> 16)
    if notificationId == noewin.CBN_SELCHANGE:
        ctrl = noeWnd.getControlByIndex(noeWnd.geomParamsCombo.comboIdx)
        comboIndex = ctrl.getSelectionIndex()

        if hasattr(noeWnd, 'currentGeometryParamsDlg'):
            if noeWnd.currentGeometryParamsDlg:
                noeWnd.currentGeometryParamsDlg.destroy()
                del noeWnd.currentGeometryParamsDlg

        if 0 == comboIndex:
            noeWnd.currentGeometryParamsDlg = GeometryParamsManualGroup(noeWnd, 30, 120)
        elif 1 == comboIndex:
            noeWnd.currentGeometryParamsDlg = GeometryParamsMatrixGroup(noeWnd, 30, 120)
        else:
            writeLog("Unknown combo index")
    return False


class GeometryParametersCombo(ControlGroupBase):
    def __init__(self, noeWnd, xpos, ypos):
        super().__init__(noeWnd, xpos, ypos)
        self.labelIdx = noeWnd.createStatic("Geometry parameters:", xpos + 0, ypos + 5, 150, 20)
        self.comboIdx = noeWnd.createComboBox(xpos + 160, ypos + 0, 90, 64, geometryParametersComboMethod)
        self.regCtrl(self.labelIdx)
        self.regCtrl(self.comboIdx)
        comboBox = noeWnd.getControlByIndex(self.comboIdx)
        comboBox.addString("Manual")
        comboBox.addString("Matrix")
        selChangCombo(comboBox)
        comboBox.selectString("Manual")


###############################################################################

class LineSpinSpinSpinGroup(ControlGroupBase):
    def __init__(self, noeWnd, xpos, ypos, paramsDict):
        super().__init__(noeWnd, xpos, ypos)

        self.edit = LabledEditGroup(noeWnd, paramsDict['editText'], str(paramsDict['editTextDef']), xpos + 0, ypos + 0, 100, 28, 100)
        self.regCtrl(self.edit)

        self.spin0Grp = createSpinBoxGroup(noeWnd, paramsDict['spin0'], xpos + 0, ypos + 32, 220, 32, 0, 128, False, 0, defaultCommandMethod, 100)
        self.regCtrl(self.spin0Grp)

        self.spin1Grp = createSpinBoxGroup(noeWnd, paramsDict['spin1'], xpos + 0, ypos + 64, 220, 32, 0, 3, False, 0, defaultCommandMethod, 100)
        self.regCtrl(self.spin1Grp)

        self.spin2Grp = createSpinBoxGroup(noeWnd, paramsDict['spin2'], xpos + 0, ypos + 96, 220, 32, 0, 3, False, 1, defaultCommandMethod, 100)
        self.regCtrl(self.spin2Grp)


# Spin|Spin
# Spin|Spin
class SpinSpinAndSpinSpinGroup(ControlGroupBase):
    def __init__(self, noeWnd, xpos, ypos, paramsDict):
        super().__init__(noeWnd, xpos, ypos)
        # Spin-Spin
        self.spin0Grp = createSpinBoxGroup(noeWnd, paramsDict['spin0'], xpos + 0, ypos + 32, 100, 32, 0, 64, False, 1, defaultCommandMethod, 100)
        self.regCtrl(self.spin0Grp)
        self.spin1Grp = createSpinBoxGroup(noeWnd, '', xpos + 110, ypos + 32, 100, 32, 0, 3, False, 0, defaultCommandMethod)
        self.regCtrl(self.spin1Grp)

        # Spin-Spin
        self.spin2Grp = createSpinBoxGroup(noeWnd, paramsDict['spin1'], xpos + 0, ypos + 64, 100, 32, 0, 64, False, 1, defaultCommandMethod, 100)
        self.regCtrl(self.spin2Grp)
        self.spin3Grp = createSpinBoxGroup(noeWnd, '', xpos + 110, ypos + 64, 100, 32, 0, 3, False, 1, defaultCommandMethod)
        self.regCtrl(self.spin3Grp)



# Spin|Spin
# Spin|Spin
# Spin|Spin
class SpinSpinAndSpinSpinAndSpinSpinGroup(ControlGroupBase):
    def __init__(self, noeWnd, xpos, ypos, paramsDict):
        super().__init__(noeWnd, xpos, ypos)
        # Spin-Spin
        self.spin0Grp = createSpinBoxGroup(noeWnd, paramsDict['spin0'], xpos + 0, ypos + 32, 100, 32, 0, 64, False, 1, defaultCommandMethod, 100)
        self.regCtrl(self.spin0Grp)
        self.spin1Grp = createSpinBoxGroup(noeWnd, '', xpos + 110, ypos + 32, 100, 32, 0, 3, False, 0, defaultCommandMethod)
        self.regCtrl(self.spin1Grp)

        # Spin-Spin
        self.spin2Grp = createSpinBoxGroup(noeWnd, paramsDict['spin1'], xpos + 0, ypos + 64, 100, 32, 0, 64, False, 1, defaultCommandMethod, 100)
        self.regCtrl(self.spin2Grp)
        self.spin3Grp = createSpinBoxGroup(noeWnd, '', xpos + 110, ypos + 64, 100, 32, 0, 3, False, 1, defaultCommandMethod)
        self.regCtrl(self.spin3Grp)

        # Spin-Spin
        self.spin4Grp = createSpinBoxGroup(noeWnd, paramsDict['spin2'], xpos + 0, ypos + 96, 100, 32, 0, 64, False, 1, defaultCommandMethod, 100)
        self.regCtrl(self.spin4Grp)
        self.spin5Grp = createSpinBoxGroup(noeWnd, '', xpos + 110, ypos + 96, 100, 32, 0, 3, False, 2, defaultCommandMethod)
        self.regCtrl(self.spin5Grp)


class TextureParamsAuto(ControlGroupBase):
    def __init__(self, noeWnd, xpos, ypos):
        super().__init__(noeWnd, xpos, ypos)
    def getOptions(self, options):
        options['attribName']= 'TEXCOORD'
        options['uvIdx']     = 0
        options['uCompIdx']  = 0
        options['vCompIdx']  = 1

class TextureParamsByAttribName(LineSpinSpinSpinGroup):
    def __init__(self, noeWnd, xpos, ypos, paramsDict):
        super().__init__(noeWnd, xpos, ypos, paramsDict)
    def getOptions(self, options):
        options['attribName'] = self.edit.getText()
        options['uvIdx']    = int(self.spin0Grp.edit.getText())
        options['uCompIdx'] = int(self.spin1Grp.edit.getText())
        options['vCompIdx'] = int(self.spin2Grp.edit.getText())

class TextureParamsByMultiAttrib(SpinSpinAndSpinSpinGroup):
    def __init__(self, noeWnd, xpos, ypos, paramsDict):
        super().__init__(noeWnd, xpos, ypos, paramsDict)
    def getOptions(self, options):
        options['uAttribIdx'] = int(self.spin0Grp.edit.getText())
        options['uCompIdx']   = int(self.spin1Grp.edit.getText())
        options['vAttribIdx'] = int(self.spin2Grp.edit.getText())
        options['vCompIdx']   = int(self.spin3Grp.edit.getText())


###############################################################################

def texturingParametersComboMethod(noeWnd, controlId, wParam, lParam):
    notificationId = (wParam >> 16)
    if notificationId == noewin.CBN_SELCHANGE:
        ctrl = noeWnd.getControlByIndex(noeWnd.texParamsCombo.comboIdx)
        comboIndex = ctrl.getSelectionIndex()
        comboStr   = ctrl.getStringForIndex(comboIndex)

        #writeLog("texturingParametersComboMethod. comboIndex={:d}".format(comboIndex))
        #writeLog("texturingParametersComboMethod. comboStr={:s}".format(comboStr))


        if hasattr(noeWnd, 'currentTexParamsDlg'):
            if noeWnd.currentTexParamsDlg:
                noeWnd.currentTexParamsDlg.destroy()
                del noeWnd.currentTexParamsDlg

        options = nrnoeimp.getImportOptions()
        if comboStr == 'Auto':
            noeWnd.currentTexParamsDlg = TextureParamsAuto(noeWnd, 30, 360)
        elif comboStr == 'By attribute name':
            options['texMode']  = 'ByAttributeName'
            paramsDict = {}
            paramsDict['editText'] = 'Attribute Name'
            paramsDict['editTextDef'] = 'TEXCOORD'
            paramsDict['spin0'] = 'UV-index'
            paramsDict['spin1'] = 'U-coord index'
            paramsDict['spin2'] = 'V-coord index'
            noeWnd.currentTexParamsDlg = TextureParamsByAttribName(noeWnd, 30, 360, paramsDict)
        elif comboStr == 'By attribute index':
            options['texMode']  = 'ByAttributeIndex'
            paramsDict = {}
            paramsDict['spin0'] = 'U attr/comp'
            paramsDict['spin1'] = 'V attr/comp'
            noeWnd.currentTexParamsDlg = TextureParamsByMultiAttrib(noeWnd, 30, 340, paramsDict)
        elif comboStr == 'By attribute name (Extra)':
            options['texMode']  = 'Extra_ByAttributeName'
            paramsDict = {}
            paramsDict['editText'] = 'Attribute Name'
            paramsDict['editTextDef'] = 'TEXCOORD'
            paramsDict['spin0'] = 'UV-index'
            paramsDict['spin1'] = 'U-coord index'
            paramsDict['spin2'] = 'V-coord index'
            noeWnd.currentTexParamsDlg = TextureParamsByAttribName(noeWnd, 30, 360, paramsDict)
        elif comboStr == 'By attribute index (Extra)':
            options['texMode']  = 'Extra_ByAttributeIndex'
            paramsDict = {}
            paramsDict['spin0'] = 'U attr/comp'
            paramsDict['spin1'] = 'V attr/comp'
            noeWnd.currentTexParamsDlg = TextureParamsByMultiAttrib(noeWnd, 30, 340, paramsDict)
        else:
            writeLog("Unknown combo index")
    return False

class TexturingParametersCombo(ControlGroupBase):
    def __init__(self, noeWnd, xpos, ypos):
        super().__init__(noeWnd, xpos, ypos)
        self.labelIdx = noeWnd.createStatic("Texturing parameters:", xpos + 0, ypos + 5, 150, 20)
        self.comboIdx = noeWnd.createComboBox(xpos + 150, ypos + 0, 200, 20, texturingParametersComboMethod)
        self.regCtrl(self.labelIdx)
        self.regCtrl(self.comboIdx)
        comboBox = noeWnd.getControlByIndex(self.comboIdx)
        comboBox.addString("Auto")
        comboBox.addString("By attribute name")
        comboBox.addString("By attribute index")
        comboBox.addString("By attribute name (Extra)")
        comboBox.addString("By attribute index (Extra)")
        selChangCombo(comboBox)
        comboBox.selectString("Auto")

###############################################################################
class NormalVectorsDisabled(ControlGroupBase):
    def __init__(self, noeWnd, xpos, ypos):
        super().__init__(noeWnd, xpos, ypos)
    def getOptions(self, options):
        return

class NormalVectorsManual(SpinSpinAndSpinSpinAndSpinSpinGroup):
    def __init__(self, noeWnd, xpos, ypos, paramsDict):
        super().__init__(noeWnd, xpos, ypos, paramsDict)
    def getOptions(self, options):
        options['norm_x_attrIdx'] = int(self.spin0Grp.edit.getText())
        options['norm_x_compIdx'] = int(self.spin1Grp.edit.getText())
        options['norm_y_attrIdx'] = int(self.spin2Grp.edit.getText())
        options['norm_y_compIdx'] = int(self.spin3Grp.edit.getText())
        options['norm_z_attrIdx'] = int(self.spin4Grp.edit.getText())
        options['norm_z_compIdx'] = int(self.spin5Grp.edit.getText())
        return

def normalParametersComboMethod(noeWnd, controlId, wParam, lParam):
    notificationId = (wParam >> 16)
    if notificationId == noewin.CBN_SELCHANGE:
        ctrl = noeWnd.getControlByIndex(noeWnd.normalVecParamsCombo.comboIdx)
        comboIndex = ctrl.getSelectionIndex()
        comboStr   = ctrl.getStringForIndex(comboIndex)

        if hasattr(noeWnd, 'currentNormalVecParamsDlg'):
            if noeWnd.currentNormalVecParamsDlg:
                noeWnd.currentNormalVecParamsDlg.destroy()
                del noeWnd.currentNormalVecParamsDlg

        options = nrnoeimp.getImportOptions()
        if comboStr == 'Disabled':
            options['normalVecMode']  = 'Disabled'
            noeWnd.currentNormalVecParamsDlg = NormalVectorsDisabled(noeWnd, 420, 330)
        elif comboStr == 'Manual':
            options['normalVecMode']  = 'Manual'
            paramsDict = {}
            paramsDict['editText'] = 'XXXAttribute Name'
            paramsDict['spin0'] = 'X attr/comp'
            paramsDict['spin1'] = 'Y attr/comp'
            paramsDict['spin2'] = 'Z attr/comp'
            noeWnd.currentNormalVecParamsDlg = NormalVectorsManual(noeWnd, 420, 330, paramsDict)
    return False

class NormalVectorCombo(ControlGroupBase):
    def __init__(self, noeWnd, xpos, ypos):
        super().__init__(noeWnd, xpos, ypos)
        self.labelIdx = noeWnd.createStatic("Normal vectors:", xpos + 10, ypos + 5, 130, 20)
        self.comboIdx = noeWnd.createComboBox(xpos + 120, ypos + 0, 150, 20, normalParametersComboMethod)
        self.regCtrl(self.labelIdx)
        self.regCtrl(self.comboIdx)
        comboBox = noeWnd.getControlByIndex(self.comboIdx)
        comboBox.addString("Disabled")
        comboBox.addString("Manual")
        selChangCombo(comboBox)
        comboBox.selectString("Disabled")

###############################################################################
def testWindowProc(hWnd, message, wParam, lParam):
    if message == noewin.WM_PAINT:
        noeWnd = noewin.getNoeWndForHWnd(hWnd)
        ps = noewin.PAINTSTRUCT()
        rect = noewin.RECT()
        hDC = noewin.user32.BeginPaint(hWnd, byref(ps))
        oldFont = None
        if noeWnd.hFont:
            oldFont = noewin.gdi32.SelectObject(hDC, noeWnd.hFont)  
        
        noewin.user32.GetClientRect(hWnd, byref(rect))
        noewin.user32.DrawTextW(hDC, "", -1, byref(rect), noewin.DT_SINGLELINE | noewin.DT_CENTER | noewin.DT_VCENTER)
        noewin.user32.EndPaint(hWnd, byref(ps))
        
        if oldFont:
            noewin.gdi32.SelectObject(hDC, oldFont)
        return 0
    return noewin.defaultWindowProc(hWnd, message, wParam, lParam)



# World space import UI
def ninjaRipperImporterGuiMethod(toolIndex):
    #noesis.logPopup()
    setRipperDirectory()

    writeLog("World space gui create")

    noeWnd = noewin.NoeUserWindow("Ninja Ripper 2 World Space Import", "NinjaRipper2WorldSpaceImportWindowClass", 800, 600, testWindowProc)
    #offset a bit into the noesis window
    noeWindowRect = noewin.getNoesisWindowRect()
    if noeWindowRect:
        windowMargin = 64
        noeWnd.x = noeWindowRect[0] + windowMargin
        noeWnd.y = noeWindowRect[1] + windowMargin
    if not noeWnd.createWindow():
        print("Failed to create window.")
        return 0

    noeWnd.setFont("Arial", 14)

    noeWnd.importType = "world"
    options = nrnoeimp.getImportOptions()
    options['importType'] = "world"
    nrnoeimp.setImportOptions(options)

    noeWnd.upControlGroup = UpControlGroup(noeWnd, 10, 10)

    # Geometry
    noeWnd.currentGeometryParamsDlg = None
    noeWnd.geomParamsCombo = GeometryParametersCombo(noeWnd, 10, 70)

    # Textures
    noeWnd.currentTexParamsDlg = None
    noeWnd.texParamsCombo = TexturingParametersCombo(noeWnd, 10, 320)
    
    # Normal
    noeWnd.currentNormalVecParamsDlg = None
    noeWnd.normalVecParamsCombo = NormalVectorCombo(noeWnd, 400, 320)
    return 0


#[[1.8106601717798212, 0.0, 0.0, 0.0], [0.0, 2.414213562373095, 0.0, 0.0], [0.0, 0.0, 1.000010000100001, 1.0], [0.0, 0.0, -0.01000010000100001, 0.0]]
def retreiveOptionsFromGui(noeWnd, options):
    if hasattr(noeWnd, 'importType'):
        if noeWnd.importType == 'world':
            # World space
            options['importType'] = "world"
            options['autoReload'] = False
            ctrl = noeWnd.getControlByIndex(noeWnd.upControlGroup.autoReloadFlagIdx)
            if ctrl and ctrl.isChecked():
                options['autoReload'] = True

            if hasattr(noeWnd, 'currentGeometryParamsDlg'):
                if isinstance(noeWnd.currentGeometryParamsDlg, GeometryParamsManualGroup):
                    options['projMat'] = noeWnd.currentGeometryParamsDlg.calcProjMat()
                elif isinstance(noeWnd.currentGeometryParamsDlg, GeometryParamsMatrixGroup):
                    options['projMat'] = nrtools.stringToMat44(noeWnd.currentGeometryParamsDlg.getMatrixStr())
                    if noeWnd.currentGeometryParamsDlg.isTransposeChecked():
                        options['projMat'] = nrtools.transposeMat(options['projMat'])
        else:
            # Local space
            options['importType'] = "local"
            if hasattr(noeWnd, 'posVecParamsDlg'):
                noeWnd.posVecParamsDlg.getOptions(options)

        # Common
        if hasattr(noeWnd, 'currentTexParamsDlg'):
            noeWnd.currentTexParamsDlg.getOptions(options)

        if hasattr(noeWnd, 'currentNormalVecParamsDlg'):
            noeWnd.currentNormalVecParamsDlg.getOptions(options)


#########################################
#########################################

class PostionVectorsManual(SpinSpinAndSpinSpinAndSpinSpinGroup):
    def __init__(self, noeWnd, xpos, ypos, paramsDict):
        super().__init__(noeWnd, xpos, ypos, paramsDict)
        self.spin0Grp.edit.setText('0')
        self.spin2Grp.edit.setText('0')
        self.spin4Grp.edit.setText('0')

    def getOptions(self, options):
        options['pos_x_attrIdx'] = int(self.spin0Grp.edit.getText())
        options['pos_x_compIdx'] = int(self.spin1Grp.edit.getText())
        options['pos_y_attrIdx'] = int(self.spin2Grp.edit.getText())
        options['pos_y_compIdx'] = int(self.spin3Grp.edit.getText())
        options['pos_z_attrIdx'] = int(self.spin4Grp.edit.getText())
        options['pos_z_compIdx'] = int(self.spin5Grp.edit.getText())
        return


# Local space import UI
def ninjaRipperImporterGuiMethodLocalSpace(toolIndex):
    #noesis.logPopup()
    setRipperDirectory()

    writeLog("Local gui create")

    noeWnd = noewin.NoeUserWindow("Ninja Ripper 2 Local Space Import", "NinjaRipper2LocalSpaceImportWindowClass", 800, 600, testWindowProc)
    #offset a bit into the noesis window
    noeWindowRect = noewin.getNoesisWindowRect()
    if noeWindowRect:
        windowMargin = 64
        noeWnd.x = noeWindowRect[0] + windowMargin
        noeWnd.y = noeWindowRect[1] + windowMargin
    if not noeWnd.createWindow():
        print("Failed to create window.")
        return 0

    noeWnd.setFont("Arial", 14)

    # If the local import window is open, then set the local import mode
    noeWnd.importType = "local"
    options = nrnoeimp.getImportOptions()
    options['importType'] = "local"
    nrnoeimp.setImportOptions(options)


    noeWnd.upControlGroup = UpControlGroupLocal(noeWnd, 10, 10)

    paramsDict = {}
    paramsDict['editText'] = 'XXXAttribute Name'
    paramsDict['spin0'] = 'PosX attr/comp'
    paramsDict['spin1'] = 'PosY attr/comp'
    paramsDict['spin2'] = 'PosZ attr/comp'
    noeWnd.posVecParamsDlg = PostionVectorsManual(noeWnd, 10, 70, paramsDict)

    # Textures
    noeWnd.currentTexParamsDlg = None
    noeWnd.texParamsCombo = TexturingParametersCombo(noeWnd, 10, 320)
    
    # Normal
    noeWnd.currentNormalVecParamsDlg = None
    noeWnd.normalVecParamsCombo = NormalVectorCombo(noeWnd, 400, 320)
    return 0
